from bigml.api import BigML
api = BigML()
source1_file = "iris.csv"
args = \
{'fields': {'000000': {'name': 'sepal length', 'optype': 'numeric'},
'000001': {'name': 'sepal width', 'optype': 'numeric'},
'000002': {'name': 'petal length', 'optype': 'numeric'},
'000003': {'name': 'petal width', 'optype': 'numeric'},
'000004': {'name': 'species',
'optype': 'categorical',
'term_analysis': {'enabled': True}}},
}
source2 = api.create_source(source1_file, args)
api.ok(source2)
args = \
{'objective_field': {'id': '000004'},
}
dataset1 = api.create_dataset(source2, args)
api.ok(dataset1)
args = \
{'anomaly_seed': 'bigml',
'seed': 'bigml'}
anomaly1 = api.create_anomaly(dataset1, args)
api.ok(anomaly1)
args = \
{}
batchanomalyscore1 = api.create_batch_anomaly_score(anomaly1, dataset1, args)
api.ok(batchanomalyscore1)


# -*- coding: utf-8 -*-
#pylint: disable=locally-disabled,unused-argument,no-member
#
# Copyright 2014-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os
import csv

from bigml.api import check_resource

from bigmler.utils import storage_file_name
from bigmler.tests.common_steps import shell_execute
from bigmler.tests.world import world, res_filename, eq_, ok_, ok_error


def i_create_all_cluster_resources(step, data=None, test=None, output=None):
    """Step: I create BigML resources uploading train <data> file to create
    centroids for <test> and log predictions in <output>
    """
    ok_(data is not None and test is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler cluster --train " + res_filename(data) +
               " --test " + test +
               " --k 8" +
               " --store --output " + output)
    shell_execute(command, output, test=test)


def i_check_create_cluster(step):
    """Step: I check that the cluster has been created"""
    cluster_file = os.path.join(world.directory, "clusters")
    message = None
    try:
        with open(cluster_file) as handler:
            cluster = check_resource(handler.readline().strip(),
                                     world.api.get_cluster)
            world.clusters.append(cluster['resource'])
            world.cluster = cluster
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def i_check_create_centroids(step):
    """Step: I check that the centroids are ready"""
    predictions_file = world.output
    with open(predictions_file, "r") as handler:
        predictions_lines = 0
        for _ in handler:
            predictions_lines += 1
    eq_(predictions_lines, world.test_lines,
        msg="predictions lines: %s, test lines: %s" % (predictions_lines,
                                                       world.test_lines))


def i_check_centroids(step, check_file):
    """Step: the local centroids file is like <check_file>"""
    check_file = res_filename(check_file)
    predictions_file = world.output
    message = None
    try:
        with open(predictions_file) as p_handler:
            predictions_file = csv.reader(p_handler, lineterminator="\n")
            with open(check_file) as c_handler:
                check_file = csv.reader(c_handler, lineterminator="\n")
        for row in predictions_file:
            check_row = next(check_file)
            message = ok_error(len(check_row) == len(row),
                               msg="Different row lengths.")
            for index, row_value in enumerate(row):
                message = ok_error(
                    row_value == check_row[index],
                    msg="Different row values: Found %s, %s expected" %
                    (row_value, check_row[index]))
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def i_create_cluster_resources_from_dataset(step, test=None, output=None):
    """Step: I create BigML resources using dataset to find centroids for
    <test> and log predictions in <output>
    """
    ok_(test is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler cluster --dataset " +
               world.dataset['resource'] + " --test " + test +  " --k 8" +
               " --store --output " + output)
    shell_execute(command, output, test=test)


def i_create_cluster_resources_from_source(step, test=None, output=None):
    """Step: I create BigML resources using source to find centroids for
    <test> and log predictions in <output>
    """
    ok_(test is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler cluster --source " +
               world.source['resource'] + " --test " + test + " --k 8" +
               " --store --output " + output)
    shell_execute(command, output, test=test)


def i_create_cluster_resources_from_local_cluster(
    step, directory=None, test=None, output=None):
    """Step: I create BigML resources using local cluster in <directory> to
    find centroids for <test> and log predictions in <output>
    """
    ok_(test is not None and output is not None and directory is not None)
    test = res_filename(test)
    with open(os.path.join(directory, "clusters")) as handler:
        cluster_id = handler.readline().strip()
    command = ("bigmler cluster --cluster-file " +
               storage_file_name(directory, cluster_id) +
               " --test " + test +
               " --store --output " + output)
    shell_execute(command, output, test=test)


def i_create_cluster_resources_from_cluster(step, test=None, output=None):
    """Step: I create BigML resources using cluster to find centroids for
    <test> and log predictions in <output>
    """
    ok_(test is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler cluster --cluster " +
               world.cluster['resource'] + " --test " + test + " --k 8" +
               " --store --output " + output)
    shell_execute(command, output, test=test)


def i_create_cluster_resources_from_clusters_file(
    step, clusters_file=None, test=None, output=None):
    """Step: I create BigML resources using clusters in file <cluster_file> to
    find centroids for <test> and log predictions in <output>
    """
    ok_(test is not None and output is not None and clusters_file is not None)
    test = res_filename(test)
    command = ("bigmler cluster --clusters " +
               clusters_file + " --test " + test +
               " --store --output " + output)
    shell_execute(command, output, test=test)


def i_create_all_cluster_resources_to_dataset(
    step, data=None, test=None, output_dir=None):
    """Step: I create BigML resources uploading train <data> file to find
    centroids for <test> remotely to dataset with no CSV and log resources in
    <output_dir>
    """
    ok_(data is not None and test is not None and output_dir is not None)
    test = res_filename(test)
    command = ("bigmler cluster --remote --train " + res_filename(data) +
               " --test " + test + " --k 8" +
               " --to-dataset --no-csv " +
               " --store --output-dir " + output_dir)
    shell_execute(command, "%s/x.csv" % output_dir, test=test)


def i_create_all_cluster_resources_with_mapping(
    step, data=None, test=None, fields_map=None, output=None):
    """Step: I create BigML resources uploading train <data> file to find
    centroids for <test> remotely with mapping file <fields_map> and log
    predictions in <output>
    """
    ok_(data is not None and test is not None and output is not None and
        fields_map is not None)
    test = res_filename(test)
    command = ("bigmler cluster --remote --train " + res_filename(data) +
               " --test " + test + " --k 8" +
               " --fields-map " + res_filename(fields_map) +
               " --store --output " + output)
    shell_execute(command, output, test=test, options="--no-header")


def i_create_all_cluster_resources_with_prediction_fields(
    step, data=None, test=None, prediction_fields=None, output=None):
    """Step: I create BigML resources uploading train <data> file to find
    centroids for <test> remotely with predictions fields <prediction_fields>
    and log predictions in <output>
    """
    ok_(data is not None and test is not None and output is not None and
        prediction_fields is not None)
    test = res_filename(test)
    command = ("bigmler cluster --remote --train " + res_filename(data) +
               " --test " + test + " --k 8" +
               " --prediction-fields \"" + prediction_fields +
               "\" --prediction-info full --prediction-header --store " +
               "--output " + output)
    shell_execute(command, output, test=test)


def i_create_cluster_from_dataset_with_summary_fields(
    step, summary_fields=None, output_dir=None):
    """Step: I create BigML cluster using dataset and summary_fields
    <summary_fields> and log resources in <output_dir>
    """
    ok_(summary_fields is not None and output_dir is not None)
    command = ("bigmler cluster --dataset " +
               world.dataset['resource'] + " --summary-fields " +
               summary_fields + " --k 8" +
               " --store --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "tmp.csv"))


def i_create_datasets_from_cluster(step, centroids=None, output=None):
    """#@step(r'I generate datasets for <centroids> centroids and log
    predictions in <output>
    """
    ok_(centroids is not None and output is not None)
    command = ("bigmler cluster --cluster " + world.cluster['resource'] +
               " --cluster-datasets \"" + centroids +
               "\" --store --output " + output)
    shell_execute(command, output, test=None)


def i_check_cluster_datasets(step, datasets_number=None):
    """Step: I check that the <datasets_number> cluster datasets are ready"""
    message = None
    try:
        datasets_file = os.path.join(world.directory, "dataset_cluster")
        with open(datasets_file) as handler:
            dataset_ids = handler.readlines()
            world.datasets.extend(dataset_ids)
            eq_(int(datasets_number), len(dataset_ids),
                msg="generated datasets %s, expected %s" % (
                    len(dataset_ids), datasets_number))
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def i_check_cluster_models(step, models_number=None):
    """Step: I check that the <models_number> cluster models are ready"""
    message = None
    try:
        models_file = os.path.join(world.directory, "models_cluster")
        with open(models_file) as handler:
            model_ids = handler.readlines()
            world.models.extend(model_ids)
            eq_(int(models_number), len(model_ids),
                msg="generated models %s, expected %s" % (
                    len(model_ids), models_number))
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def i_create_models_from_cluster(step, centroids=None, output=None):
    """Step: I generate models for <centroids> centroids and log results in
    <output>
    """
    ok_(centroids is not None and output is not None)
    command = ("bigmler cluster --dataset " + world.dataset['resource'] +
               " --cluster-models \"" + centroids +
               "\" --k 4 --store --output " + output)
    shell_execute(command, output, test=None)


def i_check_cluster_has_summary_fields(step, summary_fields=None):
    """Step: I check that the cluster has summary fields <summary_fields>"""
    eq_(world.cluster['object']['summary_fields'],
        summary_fields.split(","))



# -*- coding: utf-8 -*-
#pylint: disable=locally-disabled,unused-argument,no-member
#
# Copyright 2014-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

from bigml.api import check_resource, HTTP_NOT_FOUND

from bigmler.tests.common_steps import (
    store_init_resources, store_final_resources,
    check_init_equals_final, shell_execute)
from bigmler.tests.world import world, res_filename, ok_, eq_


def i_create_source_from_file(step, data=None, output_dir=None):
    """Step: I create a BigML source from file <data> storing results
    in <output_dir>
    """
    ok_(data is not None and output_dir is not None)
    command = ("bigmler --train " + res_filename(data) +
               " --store --output-dir " +
               output_dir + " --no-dataset --no-model --store")
    shell_execute(command, os.path.join(output_dir, "p.csv"), test=None,
                  project=False)


def i_create_project(step, name=None, output_dir=None):
    """Step: I create a BigML project with name <name> storing results in
    <output_dir>
    """
    ok_(name is not None and output_dir is not None)
    command = ("bigmler project --name " + name + " --store --output-dir " +
               output_dir)
    shell_execute(command, os.path.join(output_dir, "p.csv"), test=None,
                  project=True)


def i_check_project_exists(step):
    """Step: I check that the project exists"""
    project_file = os.path.join(world.directory, "project")
    with open(project_file) as handler:
        project_id = handler.readline().strip()
    project = check_resource(project_id,
                             world.api.get_project)
    ok_(project['code'] != HTTP_NOT_FOUND)
    world.project = project


def i_check_source_exists_by_id(step, source_id):
    """Checking that source exists"""
    source = check_resource(source_id,
                            world.api.get_source)
    ok_(source['code'] != HTTP_NOT_FOUND)
    world.source = source


def i_check_source_exists(step):
    """Step: I check that the source exists"""
    source_file = os.path.join(world.directory, "source")
    with open(source_file) as handler:
        lines = handler.readlines()
        source_id = lines[-2].strip()
    i_check_source_exists_by_id(step, source_id)


def i_check_faulty_source_exists(step):
    """Step: I check that the failed source exists"""
    i_check_source_exists(step)


def i_check_source_does_not_exist(step, source_id=None):
    """Step: I check that the source doesn't exist"""
    if source_id is None:
        source_id = world.source['resource']
    source = world.api.get_source(source_id)
    eq_(source['code'], HTTP_NOT_FOUND)


def i_check_project_does_not_exist(step):
    """Step: I check that the project doesn't exist"""
    project = world.api.get_project(world.project["resource"])
    eq_(project['code'], HTTP_NOT_FOUND)


def i_delete_source_by_ids(step, output_dir=None):
    """Step: I delete the source by id using --ids storing results
    in <output_dir>
    """
    ok_(output_dir is not None)
    command = ("bigmler delete --ids " + world.source['resource'] +
               " --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "p.csv"), test=None)


def i_delete_source_by_ids_dry(step, output_dir=None):
    """Step: I delete the source by id using --ids and --dry-run storing
    results in <output_dir>
    """
    ok_(output_dir is not None)
    command = ("bigmler delete --ids " + world.source['resource'] +
               " --dry-run --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "p.csv"), test=None)


def i_delete_source_by_ids_filtered(
    step, resource_types=None, output_dir=None):
    """Step: I delete the source by id using --ids and --resource-types
    <resource_types> storing results in <output_dir>
    """
    ok_(output_dir is not None and resource_types is not None)
    command = ("bigmler delete --ids " + world.source['resource'] +
               " --dry-run --output-dir " + output_dir +
               " --resource-types " + resource_types)
    shell_execute(command, os.path.join(output_dir, "p.csv"), test=None)


def i_delete_source_by_file(step, output_dir=None):
    """Step: I delete the source by id using --from-file and the source file
    storing results in <output_dir>
    """
    ok_(output_dir is not None)
    command = (
        "bigmler delete --from-file %s%ssource " % (output_dir, os.sep) +
        " --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "p.csv"), test=None)


def i_delete_source_faulty_by_file(step, output_dir=None):
    """Step: I delete the source by id using --from-file and --status faulty
    and the source file storing results in <output_dir>
    """
    ok_(output_dir is not None)
    command = (
        "bigmler delete --from-file %s%ssource " % (output_dir, os.sep) +
        " --status faulty --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "p.csv"), test=None)


def i_delete_source_by_file_filtered(
    step, resource_types=None, output_dir=None):
    """Step: I delete the source by id using --from-file, the source file and
    --resource-types <resource_types> storing results in <output_dir>
    """
    ok_(output_dir is not None and resource_types is not None)
    command = (
        "bigmler delete --from-file %s%ssource " % (output_dir, os.sep) +
        " --output-dir " + output_dir +
        " --resource-types " + resource_types)
    shell_execute(command, os.path.join(output_dir, "p.csv"), test=None)


def i_delete_source_older_newer(step, output_dir=None):
    """Step: I delete the source using --older-than and --newer-than storing
    results in <output_dir>
    """
    ok_(output_dir is not None)
    command = ("bigmler delete --older-than " + step.bigml["source_upper"] +
               " --newer-than " + step.bigml["source_lower"] +
               " --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "p.csv"), test=None)


def i_check_reference_source_does_not_exist(step):
    """Step: I check that the reference source doesn't exist"""
    i_check_source_does_not_exist(
        step, source_id=step.bigml["source_reference"])


def i_delete_source_older_newer_with_resource_types(
    step, resource_types=None, output_dir=None):
    """Step: I delete the source using --older-than and --newer-than with
    resource_types <resource_types> storing results in <output_dir>
    """
    ok_(output_dir is not None and resource_types is not None)
    command = ("bigmler delete --older-than " + step.bigml["source_upper"] +
               " --newer-than " + step.bigml["source_lower"] +
               " --resource-types " + resource_types +
               " --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "p.csv"), test=None)


def i_check_reference_source_exists(step):
    """Step: I check that the reference source exists"""
    i_check_source_exists_by_id(step, source_id=step.bigml["source_reference"])


def i_create_source_from_file_with_tag(
    step, data=None, tag=None, output_dir=None):
    """Step: I create a BigML source from file <data> with tag <tag> storing
    results in <output_dir>
    """
    ok_(data is not None and output_dir is not None and tag is not None)
    command = (
        "bigmler --train " + res_filename(data) + " --store --output-dir " +
        output_dir + " --tag " + tag +
        " --no-dataset --no-model --store")
    shell_execute(command, os.path.join(output_dir, "p.csv"), test=None)

def i_create_faulty_source_from_file_with_tag(
    step, data=None, tag=None, output_dir=None):
    """Step: I create a BigML source from file <data> with tag <tag> storing
    results in <output_dir>
    """
    ok_(data is not None and output_dir is not None and tag is not None)
    command = (
        "bigmler --train " + res_filename(data) + " --store --output-dir " +
        output_dir + " --tag " + tag +
        " --no-dataset --no-model --store")
    try:
        shell_execute(command, os.path.join(output_dir, "p.csv"), test=None)
    except Exception:
        pass


def i_delete_source_newer_and_tag(step, tag=None, output_dir=None):
    """Step: I delete the source using --newer-than and --source-tag
    <tag> storing results in <output_dir>
    """
    ok_(output_dir is not None and tag is not None)
    command = ("bigmler delete --newer-than " + step.bigml["source_lower"] +
               " --source-tag " + tag +
               " --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "p.csv"), test=None)


def i_delete_source_newer_faulty_and_tag(step, tag=None, output_dir=None):
    """Step: I delete the source using --newer-than and --status faulty and
    --source-tag <tag> storing results in <output_dir>
    """
    ok_(output_dir is not None and tag is not None)
    command = ("bigmler delete --newer-than " + step.bigml["source_lower"] +
               " --source-tag " + tag +
               " --status faulty --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "p.csv"), test=None)


def i_check_upper_source_exists(step):
    """Step: I check that the upper source exists"""
    i_check_source_exists_by_id(step, source_id=step.bigml["source_upper"])


def i_create_dataset_from_source_with_tag(step, tag=None, output_dir=None):
    """Step: I create a BigML dataset from the source with tag <tag> storing
    results in <output_dir>
    """
    ok_(tag is not None and output_dir is not None)
    command = ("bigmler --source " + world.source['resource'] +
               " --tag " + tag +
               " --store --output-dir " + output_dir +
               " --no-model --store")
    shell_execute(command, os.path.join(output_dir, "p.csv"), test=None)


def i_check_dataset_exists_by_id(step, dataset_id):
    """Checking that dataset exists"""
    dataset = check_resource(dataset_id,
                             world.api.get_dataset)
    ok_(dataset['code'] != HTTP_NOT_FOUND)
    world.dataset = dataset


def i_check_dataset_exists(step):
    """Step: I check that the dataset exists"""
    dataset_file = os.path.join(world.directory, "dataset")
    with open(dataset_file) as handler:
        dataset_id = handler.readline().strip()
    i_check_dataset_exists_by_id(step, dataset_id)


def i_check_dataset_does_not_exist(step, dataset_id=None):
    """Step: I check that the dataset doesn't exist$"""
    if dataset_id is None:
        dataset_id = world.dataset['resource']
    dataset = world.api.get_dataset(dataset_id)
    eq_(dataset['code'], HTTP_NOT_FOUND)


def i_delete_resources_newer_and_tag(step, tag=None, output_dir=None):
    """Step: I delete the resources using --newer-than and --all-tag <tag>
    storing results in <output_dir>
    """
    ok_(output_dir is not None and tag is not None)
    command = ("bigmler delete --newer-than " + step.bigml["source_lower"] +
               " --all-tag " + tag +
               " --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "p.csv"), test=None)


def i_create_all_resources_in_output_dir(step, data=None, output_dir=None):
    """Step: I create BigML resources uploading train <data> storing results
    in <output_dir>
    """
    ok_(output_dir is not None and data is not None)
    command = ("bigmler --train " + res_filename(data) +
               " --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "p.csv"), test=None)


def i_check_changed_number_of_resources(step):
    """Step: I check that the number of resources has changed"""
    store_final_resources()
    ok_(world.counters['sources']['final'] !=
        world.counters['sources']['init'] or
        world.counters['datasets']['final'] !=
        world.counters['datasets']['init'] or
        world.counters['models']['final'] !=
        world.counters['models']['init'] )


def i_delete_resources_from_dir(step):
    """Step: I delete the resources from the output directory"""
    command = ("bigmler delete --from-dir " + world.directory +
               " --output-dir " + world.directory)
    shell_execute(command, os.path.join(world.directory, "p.csv"), test=None)


def i_store_the_number_of_resources(step):
    """Step: I store the number of existing resources"""
    store_init_resources()


def i_check_equal_number_of_resources(step):
    """Step: the number of resources has not changed"""
    store_final_resources()
    check_init_equals_final()


def i_delete_project_by_name(step, name=None, output_dir=None):
    """Step: I delete the project by name <name>, storing results in
    <output_dir>
    """
    ok_(output_dir is not None and name is not None)
    command = ("bigmler delete --filter=\"name=%s\" "  % name +
               " --output-dir " + output_dir +
               " --resource-types project")
    shell_execute(command, os.path.join(output_dir, "p.csv"), test=None)



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#pylint: disable=locally-disabled,line-too-long,attribute-defined-outside-init
#
# Copyright 2019-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

""" Testing fusion predictions creation

"""

from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module, show_method)


import bigmler.tests.basic_tst_prediction_steps as test_pred
import bigmler.tests.basic_batch_tst_prediction_steps as batch_pred
import bigmler.tests.basic_fusion_steps as fs_pred
import bigmler.tests.evaluation_steps as evaluation


def setup_module():
    """Setup for the module

    """
    common_setup_module()
    test = TestPrediction()
    test.bigml = {"method": "setup_scenario01"}
    test.setup_scenario01()


def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()


class TestPrediction:
    """Testing fusion predictions"""

    def setup_method(self, method):
        """
            Debug information
        """
        self.bigml = {}
        self.bigml["method"] = method.__name__
        print("\n-------------------\nTests in: %s\n" % __name__)

    def teardown_method(self):
        """Calling generic teardown for every method

        """
        world.clear_paths()
        print("\nEnd of tests in: %s\n-------------------\n" % __name__)
        self.bigml = {}


    def setup_scenario01(self):
        """
        Scenario: Successfully building a model and deepnet

        """
        print(self.setup_scenario01.__doc__)
        headers = ["data", "output_dir"]
        examples = [
            ['data/iris.csv', 'scenario1_fs']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            fs_pred.setup_for_fusion(
                self, train=example["data"], output_dir=example["output_dir"])

    def test_scenario01(self):
        """
        Scenario: Successfully building test predictions from model
            And I create BigML fusion resources using preset models to test "<test>" and log predictions in "<output>"
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"

        """
        print(self.test_scenario01.__doc__)
        headers = ["test", "output", "predictions_file"]
        examples = [
            ['data/test_iris.csv', 'scenario1_fs/predictions.csv',
             'check_files/predictions_iris_fs.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            fs_pred.i_create_fs_resources_from_model(
                self, test=example["test"], output=example["output"])
            fs_pred.i_check_create_fusion(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario02(self):
        """
        Scenario: Successfully building batch test predictions from model
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML fusion resources using model built from "<train>" to test "<test>" as a batch prediction and log predictions in "<output>"
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        print(self.test_scenario02.__doc__)
        headers = ["test", "output", "predictions_file"]
        examples = [
            ['data/test_iris.csv', 'scenario2_fs/predictions.csv',
             'check_files/predictions_iris_fs2.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            fs_pred.i_create_fs_resources_from_model_remote(
                self, test=example["test"], output=example["output"])
            fs_pred.i_check_create_fusion(self)
            batch_pred.i_check_create_batch_prediction(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario03(self):
        """
        Scenario: Successfully building batch test predictions from model with customized output
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML fusion resources using model built on "<train>" to test "<test>" as a batch prediction with output format "<batch-output>" and log predictions in "<output>"
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        print(self.test_scenario03.__doc__)
        headers = ["test", "batch_output", "output", "predictions_file"]
        examples = [
            ['data/test_iris.csv', 'data/batch_output.json',
             'scenario3_fs/predictions.csv',
             'check_files/predictions_iris_fs_prob.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            fs_pred.i_create_fs_resources_from_model_remote_with_options(
                self, test=example["test"], output=example["output"],
                options_file=example["batch_output"])
            fs_pred.i_check_create_fusion(self)
            batch_pred.i_check_create_batch_prediction(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario04(self):
        """
        Scenario: Successfully building evaluation from fusion
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML fusion resources using pretrained model to test "<test>" as an evaluation and log predictions in "<output>"
            Then the evaluation file is like "<json_evaluation_file>"
        """
        print(self.test_scenario04.__doc__)
        headers = ["output", "json_evaluation_file"]
        examples = [
            ['scenario_fs_e1/evaluation',
             'check_files/evaluation_iris_fs.json']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            fs_pred.i_create_fs_resources_from_mode_and_evaluate(
                self, output=example["output"])
            fs_pred.i_check_create_fusion(self)
            test_pred.i_check_create_evaluation(self)
            evaluation.then_the_evaluation_file_is_like(
                self, example["json_evaluation_file"])



# -*- coding: utf-8 -*-
#
# Copyright 2012-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Utilities for BigMLer

"""


import fileinput
import ast
import glob
import os
import sys
import datetime

try:
    import simplejson as json
except ImportError:
    import json

import bigml.api
from bigml.constants import EXTERNAL_CONNECTION_ATTRS
from bigml.util import console_log, empty_resource
from bigml.fields import get_fields_structure, Fields
from bigml.io import UnicodeReader

PAGE_LENGTH = 200
ATTRIBUTE_NAMES = ['name', 'label', 'description']
NEW_DIRS_LOG = ".bigmler_dirs"
BRIEF_MODEL_QS = "exclude=root,fields"

# Base Domain
BIGML_DOMAIN = os.environ.get('BIGML_DOMAIN', 'bigml.io')
BIGML_DASHBOARD_URL = os.environ.get('BIGML_DASHBOARD_URL')
BIGML_DASHBOARD_DOMAIN = (
    "%s.com" % BIGML_DOMAIN[:-3] if BIGML_DOMAIN[-3:] == '.io' else
    BIGML_DOMAIN.replace("-io.", "."))
RESOURCE_URL = ("https://%s/dashboard/" % BIGML_DASHBOARD_DOMAIN
                if BIGML_DASHBOARD_URL is None
                else BIGML_DASHBOARD_URL)
RESOURCE_SHARED_URL = "%s/shared/" % "/".join(RESOURCE_URL.split('/')[:-2])
RESOURCE_EMBEDDED_URL = ("%s/embedded/%%s/tree" %
                         "/".join(RESOURCE_URL.split('/')[:-2]))
BIGML_SYS_ENCODING = os.environ.get('BIGML_SYS_ENCODING', \
    sys.getfilesystemencoding() if sys.platform == 'win32' else 'utf-8')
FILE_ENCODING = 'utf-8'
# resources that can have other resources as components
COMPOSED_RESOURCES = [
    "cluster", "ensemble", "fusion", "composites", "optiml",
    "evaluation", "timeseries", "source"]


def read_description(path):
    """Reads a text description from a file.

    """
    return ''.join(fileinput.input([path]))


def read_field_attributes(path):
    """Reads field attributes from a csv file to update source fields.

    A column number and a list of attributes separated by a comma per line.
    The expected structure is:
    column number, name, label, description

    For example:

    0,'first name','label for the first field','fist field full description'
    1,'last name','label for the last field','last field full description'

    """
    field_attributes = {}
    try:
        with UnicodeReader(path, quotechar="'") as attributes_reader:
            for row in attributes_reader:
                attributes = {}
                if len(row) > 1:
                    for index in range(0, min(len(ATTRIBUTE_NAMES),
                                              len(row) - 1)):
                        attributes.update(
                            {ATTRIBUTE_NAMES[index]: row[index + 1]})
                    field_attributes.update({
                        int(row[0]): attributes})
            return field_attributes
    except IOError:
        sys.exit("Error: cannot read field attributes %s" % path)


def read_types(path):
    """Types to update source fields types.

    A column number and type separated by a comma per line.

    For example:

    0, 'categorical'
    1, 'numeric'

    """
    types_dict = {}
    for line in fileinput.input([path]):
        try:
            pair = ast.literal_eval(line)
            types_dict.update({
                pair[0]: {'optype': pair[1]}})
        except SyntaxError:
            console_log("WARNING: The following line in file %s"
                        " does not match the expected"
                        " syntax: %s" % (path, line))
    return types_dict


def read_json(path):
    """Reads attributes from a json file to update or create resources.
       For instance, to change the text analysis mode to full text in the
       field whose id is '000001'

        {"fields": {
            "000001": {
                "term_analysis": {
                    "token_mode": "full_terms_only"}}}}

    """
    json_attributes = {}
    try:
        with open(path, open_mode('r')) as attributes_reader:
            json_attributes = json.load(attributes_reader)
    except IOError:
        sys.exit("Error: cannot read json file %s" % path)
    except ValueError:
        sys.exit("Error: no valid json found in %s" % path)
    return json_attributes


def read_fields_map(path):
    """Fields map from evaluated model to test dataset.

    The the evaluated model field column and the test dataset field column
    separated by a comma per line.

    For example:

    0, 0
    1, 2
    2, 1

    """
    map_dict = {}
    for line in fileinput.input([path]):
        try:
            pair = ast.literal_eval(line)
            map_dict.update({
                pair[0]: pair[1]})
        except SyntaxError:
            console_log("WARNING: The following line in file %s"
                        " does not match the expected"
                        " syntax: %s" % (path, line))
    return map_dict


def read_resources(path):
    """Reads resources ids from a file.

    For example:

    model/50974922035d0706da00003d
    model/509748b7035d0706da000039
    model/5097488b155268377a000059

    """
    resources = []
    for line in fileinput.input([path]):
        resources.append(line.rstrip())
    return resources


def read_datasets(path):
    """Reads dataset ids from a file.

    For example:

    dataset/50978822035d0706da000069
    dataset/50978822035d0706da000073
    dataset/50978822035d0706da000085

    """
    datasets = []
    for line in fileinput.input([path]):
        datasets.append(line.rstrip())
    return datasets


def read_sources(path, read_all=False):
    """Reads last source id from a file.

    """
    sources = []
    for line in fileinput.input([path]):
        if line.rstrip().startswith("source/"):
            sources.append(line.rstrip())
    if read_all:
        return sources
    try:
        return sources[-1]
    except KeyError:
        return None


def read_json_filter(path):
    """Reads a json filter from a file.

    For example:

    [">", 3.14, ["field", "000002"]]

    """
    with open(path) as json_data:
        return json.load(json_data)


def read_lisp_filter(path):
    """Reads a lisp filter from a file.

    For example:

    (> (/ (+ (- (field "00000") 4.4)
            (field 23)
            (* 2 (field "Class") (field "00004")))
       3)
       5.5)

    """
    return read_description(path)


def read_votes_files(dirs_list, path):
    """Reads a list of directories to look for votes.

    If model's prediction files are found, they are retrieved to be combined.
    Models' predictions files are expected to be named after the model id,
    for instance: model_50974922035d0706da00003d__predictions.csv
    """
    file_name = "%s%scombined_predictions" % (path, os.sep)
    check_dir(file_name)
    with open(file_name, "wb", 0) as group_predictions:
        current_directory = os.getcwd()
        predictions_files = []
        for directory in dirs_list:
            directory = os.path.abspath(directory)
            os.chdir(directory)
            for predictions_file in glob.glob("model_*_predictions.csv"):
                predictions_files.append("%s%s%s" % (os.getcwd(),
                                                     os.sep, predictions_file))
                message = "%s\n" % predictions_file
                message = message.encode(FILE_ENCODING)
                group_predictions.write(message)
            os.chdir(current_directory)
    return predictions_files


def read_local_resource(path, csv_properties=None):
    """Read the JSON resource structure information from the given file.

    """
    resource = empty_resource()
    if csv_properties is None:
        csv_properties = {}
    fields = None
    with open(path, open_mode('r')) as resource_file:
        try:
            resource = json.loads(resource_file.read())
        except IOError:
            pass
    resource_id = resource.get('resource')
    if resource_id is None:
        sys.exit("Failed to extract a BigML resource structure from the"
                 " contents of file %s." % path)
    if resource.get('object') is None:
        resource = {'resource': resource_id,
                    'object': resource,
                    'error': None,
                    'code': bigml.api.HTTP_OK}

    fields, resource_locale, missing_tokens, \
    objective_column = get_fields_structure(resource)
    if 'objective_field' not in csv_properties and \
            objective_column is not None:
        csv_properties['objective_field'] = objective_column
    if missing_tokens:
        csv_properties['missing_tokens'] = missing_tokens
    if resource_locale:
        csv_properties['data_locale'] = resource_locale
    if fields:
        fields = Fields(resource, **csv_properties)
    return resource, csv_properties, fields


def list_ids(api_function, query_string, status_code=bigml.api.FINISHED,
             limit=None):
    """Lists BigML resources filtered by `query_string`.

    """
    if status_code is None:
        status_str = ""
    else:
        status_str = 'status.code=%s&' % status_code
    limit_s = 'limit=%s' % (PAGE_LENGTH if limit is None else limit)
    q_s = '%s%s&%s' % (
        status_str, limit_s, query_string)
    resources = api_function(q_s)
    ids = [obj['resource'] for obj in (resources['objects'] or [])]
    while (resources['objects'] and (limit is None or len(ids) < limit) and
           (resources['meta']['total_count'] > (resources['meta']['offset'] +
                                                resources['meta']['limit']))):
        offset = resources['meta']['offset'] + PAGE_LENGTH
        q_s = '%soffset=%s&%s&%s' % (
            status_str, offset, limit_s, query_string)
        resources = api_function(q_s)
        if resources['objects']:
            ids.extend([obj['resource'] for obj in resources['objects']])
    return ids


def delete(api, delete_list, exe_outputs=True, query_string=''):
    """ Deletes the resources given in the list. If the exe_outputs is set,
        deleting an execution causes the deletion of any outpur resource.

    """
    for resource_id in delete_list:
        resource_type = None
        try:
            for resource_type in bigml.api.RESOURCE_RE:
                try:
                    bigml.api.get_resource(resource_type, resource_id)
                    break
                except ValueError:
                    pass
            query_string_list = [query_string]
            if (resource_type == "execution" and exe_outputs) or \
                    resource_type in COMPOSED_RESOURCES:
                query_string_list.append("delete_all=true")
            api.deleters[resource_type](
                resource_id,
                query_string="&".join(query_string_list))
        except ValueError:
            console_log("Failed to delete resource %s" % resource_id)


def check_dir(path):
    """Creates a directory if it doesn't exist

    """
    directory = os.path.dirname(path)
    if directory == "":
        directory = "."
    if directory and not os.path.exists(directory):
        os.makedirs(directory)
        sys_log_message("%s\n" % os.path.abspath(directory),
                        log_file=NEW_DIRS_LOG)

    return directory


#pylint: disable=locally-disabled,anomalous-backslash-in-string
def print_tree(directory, padding):
    """Returns a graphical directory tree structure as a string

    """
    connectors = ['+--', '\\--'] if sys.platform == 'win32' else ['├─', '└─']
    if padding != ' ':
        output = padding[:-1] + connectors[0]
    else:
        output = padding
    output += os.path.basename(os.path.abspath(directory)) + '\n'
    padding = padding + ' '
    files = []
    files = os.listdir(directory)
    count = 0
    for i, file_name in enumerate(files):
        count += 1
        path = directory + os.sep + file_name
        if os.path.isdir(path):
            if count == len(files):
                output += print_tree(path, padding + ' ')
            else:
                output += print_tree(path, padding + '|')
        else:
            if i < (len(files) - 1):
                output += padding + connectors[0] + file_name + '\n'
            else:
                output += padding + connectors[1] + file_name + '\n'
    output = os.path.dirname(os.path.abspath(directory)) + ':\n' + \
        "    " + output.replace("\n", "\n    ")
    return output


def dated(message):
    """Prepends date in log format in string

    """
    return "[%s] %s" % (datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
                        message)


def get_url(resource, shared=False, embedded=False):
    """Returns the resource's url in bigml.com

    """
    if shared:
        return (RESOURCE_SHARED_URL + bigml.api.get_resource_type(resource)
                + os.sep + resource['object']['shared_hash'])
    if embedded:
        return (RESOURCE_EMBEDDED_URL % (bigml.api.get_resource_type(resource)
                                         + os.sep +
                                         resource['object']['shared_hash']))
    resource_id = bigml.api.get_resource_id(resource)
    if not resource_id:
        return ""
    return RESOURCE_URL + resource_id


def log_message(message, log_file=None, console=False):
    """Logs a message in a file and/or to console

       If log_file is set, logs the message in the file.
       If console is True, sends the message to console.
    """

    if console:
        console_log(message)
    if log_file is not None:
        message = message.encode(FILE_ENCODING)
        with open(log_file, 'ab', 0) as log_handler:
            log_handler.write(message)


def sys_log_message(message, log_file=None, mode='ab'):
    """Logs a message in a file using the system encoding

       If log_file is set, logs the message in the file.
    """
    message = message.encode(BIGML_SYS_ENCODING)
    if not mode.endswith("b"):
        mode = "%sb" % mode
    if log_file is not None:
        with open(log_file, mode) as log_handler:
            log_handler.write(message)


def plural(text, num):
    """Pluralizer: adds "s" at the end of a string if a given number is > 1

    """
    return "%s%s" % (text, "s"[num == 1:])


def check_resource_error(resource, message):
    """If a given resource is faulty, or some other error has occured, exits.
       Returns the resource id otherwise.

    """
    if ('error' in resource and resource['error'] or
            bigml.api.get_status(resource)['code'] == bigml.api.FAULTY):
        error_message = ""
        try:
            if ('status' in resource['error'] and
                    'message' in resource['error']['status']):
                error_message = resource['error']['status']['message']
        except TypeError:
            status = bigml.api.get_status(resource)
            if status.get('error') is not None:
                error_message = status["message"]
        sys.exit("%s%s" % (message, error_message))
    return bigml.api.get_resource_id(resource)


def log_created_resources(file_name, path, resource_id, mode='ab',
                          comment=None):
    """Logs the created resources ids in the given file

    """
    if path is not None:
        file_name = "%s%s%s" % (path, os.sep, file_name)
        try:
            message = ""
            if resource_id is not None:
                message = "%s\n" % resource_id
            if comment is not None:
                message = "%s%s" % (message, comment)
            sys_log_message(message, file_name, mode)
        except IOError as exc:
            print("Failed to write %s: %s" % (file_name, str(exc)))


def check_resource(*args, **kwargs):
    """Wrapper to catch errors in resource retrieval

    """
    try:
        kwargs.update({"raise_on_error": True})
        result = bigml.api.check_resource(*args, **kwargs)
        return result
    except Exception as exc:
        sys.exit("\nFailed to obtain a finished resource:\n%s." % str(exc))


def objective_field_names(models_or_ensembles, api):
    """Gets the objective field names for a list of models or ensembles

    """
    objective_fields = []
    for model_or_ensemble in models_or_ensembles:
        name = None
        if isinstance(model_or_ensemble, dict):
            status_code = bigml.api.get_status(model_or_ensemble)['code']
            if status_code == bigml.api.FINISHED:
                name = objective_field_name(model_or_ensemble, api)
        if name is None:
            resource_type = bigml.api.get_resource_type(model_or_ensemble)
            if resource_type == bigml.api.MODEL_PATH:
                model_id = bigml.api.get_model_id(model_or_ensemble)
                if model_id is not None:
                    try:
                        model = check_resource(model_id, api.get_model,
                                               query_string=BRIEF_MODEL_QS)
                    except ValueError as exception:
                        sys.exit("Failed to get a finished model: %s" %
                                 str(exception))
                    name = objective_field_name(model, api)
            elif resource_type == bigml.api.ENSEMBLE_PATH:
                ensemble_id = bigml.api.get_ensemble_id(model_or_ensemble)
                if ensemble_id is not None:
                    try:
                        ensemble = check_resource(
                            ensemble_id, api.get_ensemble)
                    except ValueError as exception:
                        sys.exit("Failed to get a finished ensemble: %s" %
                                 str(exception))
                    name = objective_field_name(ensemble, api)
            else:
                sys.exit("No valid model or ensemble id.")
        if name is not None:
            objective_fields.append(name)

    return objective_fields


def objective_field_name(model_or_ensemble, api):
    """Gets the objective field name from a dict model or ensemble structure

    """
    if not isinstance(model_or_ensemble, dict):
        return None
    if not 'object' in model_or_ensemble:
        return None
    objective_field = None
    resource_info = model_or_ensemble['object']
    if 'objective_field' in resource_info:
        objective_field = resource_info['objective_field']
    elif 'objective_fields' in resource_info:
        objective_field = resource_info['objective_fields'][0]
    if objective_field is None:
        return None
    resource_type = bigml.api.get_resource_type(model_or_ensemble)
    if resource_type == bigml.api.ENSEMBLE_PATH:
        bigml.api.get_ensemble_id(model_or_ensemble)
        model = model_or_ensemble['object']['models'][0]
        model = api.get_model(model, query_string=BRIEF_MODEL_QS)
    elif resource_type == bigml.api.MODEL_PATH:
        model = model_or_ensemble
    else:
        sys.exit("No valid model or ensemble structure.")
    return model['object']['model']['model_fields'][objective_field]['name']


def read_objective_weights(path):
    """Reads objective weights from a CSV file in a class, weight format.

    The expected structure is:
    class name, weight

    For example:

    Iris-setosa,5
    Iris-versicolor,10

    """
    objective_weights = []
    try:
        with UnicodeReader(path, quotechar="'") as weights_reader:
            for row in weights_reader:
                weights = []
                if len(row) != 2:
                    sys.exit("Error: wrong objective field file syntax\n%s" %
                             ",".join(row))
                weights = row[:]
                try:
                    weights[1] = int(weights[1])
                except ValueError:
                    sys.exit("Error: wrong objective field file syntax\n%s" %
                             ",".join(row))
                objective_weights.append(weights)
            return objective_weights
    except IOError:
        sys.exit("Error: cannot read objective weights %s" % path)


def is_shared(resource):
    """Checks if a resource is shared

    """
    return resource['object'].get('shared', False)


def get_options_list(args, options, prioritary=None):
    """Creates the list of values that generates the options in `options`
       from the args object.

    """
    options_list = []
    filtered_options = options[:]
    non_equivalent_options = {'dev': 'dev_mode'}
    # removes the options that are already set as prioritary
    for element in prioritary:
        try:
            filtered_options.remove(element)
        except ValueError:
            pass
    # adds the options that have not been assigned a value in prioritary
    for option in filtered_options:
        try:
            flag = option[2:].replace("-", "_")
            if flag in list(non_equivalent_options.keys()):
                flag = non_equivalent_options[flag]
            value = getattr(args, flag)
            if value is not None:
                if isinstance(value, bool):
                    if value:
                        options_list.append(option)
                elif isinstance(value, list):
                    if value:
                        options_list.append("%s=%s" %
                                            (option, ",".join(value)))
                else:
                    if not isinstance(value, str):
                        value = str(value)
                    options_list.append("%s=%s" % (option, value))
        except AttributeError:
            pass
    return options_list


def print_generated_files(path, log_file=None, verbosity=1):
    """Prints the file structure generated while running bigmler

    """
    message = ("\nGenerated files:\n\n" +
               print_tree(path, " ") + "\n")
    log_message(message, log_file=log_file, console=verbosity)


def storage_file_name(directory, resource_id):
    """Returns the path to the JSON file that contains a stored resource
       structure

    """
    return os.path.normpath(
        os.path.join(directory, resource_id.replace("/", "_")))


def get_objective_id(fields, objective):
    """Checks if the objective given by the user in the --objective flag
       is in the list of fields. Returns its column number or None otherwise.

    """
    if objective is None:
        return None
    try:
        objective_id = fields.field_id(objective)
    except ValueError:
        return None
    return objective_id


def open_mode(mode):
    """Python 3 compatible open mode

    """
    return "%st" % mode


def transform_fields_keys(json_attributes, fields):
    """Transforms the fields structure keys if they are expressed in columns
       to the corresponding IDs

    """
    fields_structure = {}
    if fields is None:
        return json_attributes
    if "fields" in list(json_attributes.keys()):
        old_keys = list(json_attributes["fields"].keys())
        for old_key in old_keys:
            try:
                if not old_key in fields.fields:
                    key = fields.field_id(int(old_key))
                else:
                    key = old_key
            except ValueError:
                key = old_key
            fields_structure[key] = json_attributes["fields"][old_key]
        json_attributes["fields"] = fields_structure
    return json_attributes


def get_last_resource(resource_type, api=None, query_string=None):
    """Retrieves the last resource that meets the conditions in args

    """
    if query_string is None:
        query_string = ""
    if api is None:
        api = bigml.api.BigML()
    ids = list_ids(api.listers[resource_type], query_string, limit=1)
    if ids:
        return ids[0]
    return None


def get_first_resource(resource_type, api=None, query_string=None):
    """Retrieves the first resource that meets the conditions in args

    """
    order = "order_by=created"
    if query_string is None:
        query_string = order
    else:
        query_string = "%s&%s" % (query_string, order)

    if api is None:
        api = bigml.api.BigML()
    ids = list_ids(api.listers[resource_type], query_string, limit=1)
    if ids:
        return ids[0]
    return None


def last_resource_url(resource_id, api=None, query_string=None):
    """Creates the last resource URL of the given type
    that meets the conditions in args

    """
    if query_string is None:
        query_string = ""
    if api is None:
        api = bigml.api.BigML()
    auth = "%sproject=%s&" % (api.auth, api.project) if api.project else api.auth
    resource_type = bigml.api.get_resource_type(resource_id)
    return "%s%s%s%s" % (api.url, resource_type, auth, query_string)


def get_script_id(path):
    """Returns the script id stored in the file in path

    """
    try:
        with open(path) as file_handler:
            return file_handler.read().strip()
    except IOError:
        return None


def add_api_context(command_args, args):
    """Extends the command_list with the arguments related to the api
       credentials.

    """
    if args.org_project:
        command_args.extend(['--org-project', args.org_project])
    if args.username:
        command_args.extend(['--username', args.username])
    if args.api_key:
        command_args.extend(['--api-key', args.api_key])


def write_to_utf8(path, text):
    """Encoded UTF-8 text write

    """
    with open(path, "w", encoding="utf-8") as file_handler:
        file_handler.write(text)


def has_connection_info(args):
    """Checks the information needed to create a connector

    """
    check = True
    connection_env_vars = list(EXTERNAL_CONNECTION_ATTRS.keys())

    for key in connection_env_vars:
        value = EXTERNAL_CONNECTION_ATTRS[key]
        # source has a default
        if value == "source":
            continue
        check = check and hasattr(args, value) and getattr(args, value)
        if not check:
            # try environment variables
            check = os.environ.get(key) is not None
            if not check:
                break

    return check



# -*- coding: utf-8 -*-
#pylint: disable=locally-disabled,unused-argument,no-member
#
# Copyright 2014-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import os
import json

from bigmler.tests.common_steps import shell_execute
from bigmler.tests.world import world, res_filename, ok_


def i_create_all_ml_evaluations(
    step, tag=None, label_separator=None, number_of_labels=None, data=None,
    training_separator=None, output=None):
    """Step: I create BigML multi-label resources tagged as <tag> with
    <label_separator> label separator and <number_of_labels> labels uploading
    train <data> file with <training_separator> field separator to evaluate
    and log evaluation in <output>
    """
    ok_(tag is not None and label_separator is not None and
        training_separator is not None and number_of_labels is not None
        and data is not None and output is not None)
    world.number_of_models = int(number_of_labels)
    command = ("bigmler --multi-label --train " + res_filename(data) +
        " --label-separator \"" + label_separator +
        "\" --training-separator \"" + training_separator +
        "\" --evaluate --store --output " + output +
        " --tag " + tag + " --max-batch-models 1")
    shell_execute(command, output)


def i_create_all_ml_resources_for_evaluation(
    step, tag=None, label_separator=None, number_of_labels=None, data=None,
    training_separator=None, output=None):
    """Step: I create BigML multi-label resources tagged as <tag> with
    <label_separator> label separator and <number_of_labels> labels uploading
    train <data> file with <training_separator> field separator to evaluate
    and log in <output>
    """
    ok_(tag is not None and label_separator is not None and
        training_separator is not None and number_of_labels is not None
        and data is not None and output is not None)
    world.number_of_models = int(number_of_labels)
    command = ("bigmler --multi-label --train " + res_filename(data) +
        " --label-separator \"" + label_separator +
        "\" --training-separator \"" + training_separator +
        "\" --store --output " + output +
        " --tag " + tag + " --max-batch-models 1")
    shell_execute(command, output)


def i_check_evaluation_ready(step):
    """Step: I check that the evaluation is ready"""
    evaluation_file = os.path.join(world.directory, "evaluation.json")
    message = None
    try:
        with open(evaluation_file) as handler:
            json.loads(handler.readline().strip())
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def i_create_ml_evaluations_from_source(step, output=None):
    """Step: I create BigML <multi-label> resources using source to evaluate
    and log evaluation in <output>
    """
    ok_(output is not None)
    command = ("bigmler --multi-label --source " + world.source['resource'] +
        " --evaluate --store --output " + output)
    shell_execute(command, output)


def i_create_ml_evaluations_from_dataset(step, output=None):
    """Step: I create BigML <multi-label> resources using dataset to evaluate
    and log evaluation in <output>
    """
    ok_(output is not None)
    command = ("bigmler --multi-label --dataset " +
               world.dataset['resource'] + " --evaluate --store --output "
               + output)
    shell_execute(command, output)


def i_create_ml_evaluations_from_models(step, models_file=None, output=None):
    """Step: I create BigML <multi-label> resources using models in file
    <models_file> to evaluate and log evaluation in <output>
    """
    ok_(output is not None and models_file is not None)
    command = ("bigmler --multi-label --models " + models_file +
               " --dataset " + world.dataset['resource'] +
               " --evaluate --store --output " + output)
    shell_execute(command, output)


def i_create_ml_evaluations_from_tagged_models(step, tag=None, output=None):
    """Step: I create BigML <multi-label> resources using models tagged as
    <tag> to evaluate and log evaluation in <output>
    """
    ok_(output is not None and tag is not None)
    command = ("bigmler --multi-label --model-tag " + tag + " --dataset " +
               world.dataset['resource'] + " --evaluate --store --output "
               + output)
    shell_execute(command, output)



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#pylint: disable=locally-disabled,line-too-long,attribute-defined-outside-init
#
# Copyright 2015-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Testing Anomaly Detector scores

"""

from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module, show_method)

import bigmler.tests.basic_tst_prediction_steps as test_pred
import bigmler.tests.basic_anomaly_prediction_steps as test_anomaly



def setup_module():
    """Setup for the module

    """
    common_setup_module()
    test = TestAnomaly()
    test.bigml = {"method": "setup_scenario1"}
    test.setup_scenario1()

def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()


class TestAnomaly:
    """Testing anomaly scores"""

    def setup_method(self, method):
        """Debug information """
        self.bigml = {}
        self.bigml["method"] = method.__name__
        print("\n-------------------\nTests in: %s\n" % __name__)

    def teardown_method(self):
        """Calling generic teardown for every method

        """
        world.clear_paths()
        print("\nEnd of tests in: %s\n-------------------\n" % __name__)
        self.bigml = {}

    def setup_scenario1(self):
        """
        Scenario: Successfully building test anomaly scores from scratch:
            Given I create BigML resources uploading train "<data>" file to create anomaly scores for "<test>" and log predictions in "<output>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the anomaly detector has been created
            And I check that the anomaly scores are ready
            Then the local anomaly scores file is like "<predictions_file>"
        """
        print(self.setup_scenario1.__doc__)
        headers = ["data", "test", "output", "predictions_file"]
        examples = [
            ['data/tiny_kdd.csv', 'data/test_kdd.csv', 'scenario_an_1/anomaly_scores.csv', 'check_files/anomaly_scores_kdd.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_anomaly.i_create_all_anomaly_resources(
                self, data=example["data"], test=example["test"],
                output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self)
            test_anomaly.i_check_create_anomaly(self)
            test_anomaly.i_check_create_anomaly_scores(self)
            test_anomaly.i_check_anomaly_scores(
                self, example["predictions_file"])

    def test_scenario2(self):
        """
        Scenario: Successfully building test predictions from source
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML resources using source to find anomaly scores for "<test>" and log predictions in "<output>"
            And I check that the dataset has been created
            And I check that the anomaly detector has been created
            And I check that the anomaly scores are ready
            Then the local anomaly scores file is like "<predictions_file>"
        """
        print(self.test_scenario2.__doc__)
        headers = ["scenario", "kwargs", "test", "output", "predictions_file"]
        examples = [
            ['scenario_an_1', '{"data": "data/tiny_kdd.csv",' +
             ' "output": "scenario_an_1/anomaly_scores.csv",' +
             ' "test": "data/test_kdd.csv"}', 'data/test_kdd.csv',
             'scenario_an_2/anomaly_scores.csv',
             'check_files/anomaly_scores_kdd.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_anomaly.i_create_anomaly_resources_from_source(
                self, test=example["test"], output=example["output"])
            test_pred.i_check_create_dataset(self)
            test_anomaly.i_check_create_anomaly(self)
            test_anomaly.i_check_create_anomaly_scores(self)
            test_anomaly.i_check_anomaly_scores(
                self, example["predictions_file"])

    def test_scenario3(self):
        """
        Scenario: Successfully building test predictions from dataset
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML resources using dataset to find anomaly scores for "<test>" and log predictions in "<output>"
            And I check that the anomaly detector has been created
            And I check that the anomaly scores are ready
            Then the local anomaly scores file is like "<predictions_file>"
        """
        print(self.test_scenario3.__doc__)
        headers = ["scenario", "kwargs", "test", "output", "predictions_file"]
        examples = [
            ['scenario_an_1', '{"data": "data/tiny_kdd.csv",' +
             ' "output": "scenario_an_1/anomaly_scores.csv",' +
             ' "test": "data/test_kdd.csv"}', 'data/test_kdd.csv',
             'scenario_an_3/anomaly_scores.csv',
             'check_files/anomaly_scores_kdd.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_anomaly.i_create_anomaly_resources_from_dataset(
                self, test=example["test"], output=example["output"])
            test_anomaly.i_check_create_anomaly(self)
            test_anomaly.i_check_create_anomaly_scores(self)
            test_anomaly.i_check_anomaly_scores(
                self, example["predictions_file"])

    def test_scenario4(self):
        """
            Scenario: Successfully building test predictions from anomaly
                Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
                And I create BigML resources using anomaly detector to find anomaly scores for "<test>" and log predictions in "<output>"
                And I check that the anomaly scores are ready
                Then the local anomaly scores file is like "<predictions_file>"
        """
        print(self.test_scenario4.__doc__)
        headers = ["scenario", "kwargs", "test", "output", "predictions_file"]
        examples = [
            ['scenario_an_1', '{"data": "data/tiny_kdd.csv",' +
             ' "output": "scenario_an_1/anomaly_scores.csv",' +
             ' "test": "data/test_kdd.csv"}', 'data/test_kdd.csv',
             'scenario_an_4/anomaly_scores.csv',
             'check_files/anomaly_scores_kdd.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_anomaly.i_create_anomaly_resources_from_anomaly_detector(
                self, test=example["test"], output=example["output"])
            test_anomaly.i_check_create_anomaly_scores(self)
            test_anomaly.i_check_anomaly_scores(
                self, example["predictions_file"])

    def test_scenario5(self):
        """
            Scenario: Successfully building test predictions from anomaly detector file
                Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
                And I create BigML resources using anomaly detector in file "<anomaly_file>" to find anomaly scores for "<test>" and log predictions in "<output>"
                And I check that the anomaly scores are ready
                Then the local anomaly scores file is like "<predictions_file>"
        """
        print(self.test_scenario5.__doc__)
        headers = ["scenario", "kwargs", "anomaly_file", "test", "output",
                   "predictions_file"]
        examples = [
            ['scenario_an_1', '{"data": "data/tiny_kdd.csv",' +
             ' "output": "scenario_an_1/anomaly_scores.csv",' +
             ' "test": "data/test_kdd.csv"}', 'scenario_an_1/anomalies',
             'data/test_kdd.csv', 'scenario_an_5/anomaly_scores.csv',
             'check_files/anomaly_scores_kdd.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_anomaly.i_create_anomaly_resources_from_anomaly_file(
                self, anomaly_file=example["anomaly_file"],
                test=example["test"], output=example["output"])
            test_anomaly.i_check_create_anomaly_scores(self)
            test_anomaly.i_check_anomaly_scores(
                self, example["predictions_file"])

    def test_scenario6(self):
        """
        Scenario: Successfully building test predictions from anomaly
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML resources using local anomaly detector in "<scenario>" to find anomaly scores for "<test>" and log predictions in "<output>"
            And I check that the anomaly scores are ready
            Then the local anomaly scores file is like "<predictions_file>"
        """
        print(self.test_scenario6.__doc__)
        headers = ["scenario", "kwargs", "test", "output", "predictions_file"]
        examples = [
            ['scenario_an_1', '{"data": "data/tiny_kdd.csv",' +
             ' "output": "scenario_an_1/anomaly_scores.csv",' +
             ' "test": "data/test_kdd.csv"}', 'data/test_kdd.csv',
             'scenario_an_6/anomaly_scores.csv',
             'check_files/anomaly_scores_kdd.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_anomaly.i_create_anomaly_resources_from_local_anomaly_detector(
                self, example["scenario"], test=example["test"],
                output=example["output"])
            test_anomaly.i_check_create_anomaly_scores(self)
            test_anomaly.i_check_anomaly_scores(
                self, example["predictions_file"])

    def test_scenario7(self):
        """
        Scenario: Successfully building anomalous dataset test predictions from anomaly
            Given I create BigML anomaly detector from data <data> with options <options> and generate a new dataset of anomalies in "<output_dir>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the anomaly detector has been created
            Then I check that the new top anomalies dataset has been created
            And the top anomalies in the anomaly detector are <top_anomalies>
            And the forest size in the anomaly detector is <forest_size>
            And the number of records in the top anomalies dataset is <top_anomalies>
        """
        print(self.test_scenario7.__doc__)
        headers = ["data", "options", "output_dir", "top_anomalies",
                   "forest_size"]
        examples = [
            ['data/tiny_kdd.csv', '--top-n 15 --forest-size 40 ',
             'scenario_an_7', '15', '40']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_anomaly.i_create_anomaly_resources_with_options(
                self, example["data"], example["options"],
                output_dir=example["output_dir"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self)
            test_anomaly.i_check_create_anomaly(self)
            test_pred.i_check_create_dataset(self, suffix='gen ')
            test_anomaly.i_check_top_anomalies(self, example["top_anomalies"])
            test_anomaly.i_check_forest_size(self, example["forest_size"])
            test_anomaly.i_check_dataset_lines_number(
                self, example["top_anomalies"])

    def test_scenario8(self):
        """
        Scenario: Successfully building anomaly detector from dataset with id fields
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create anomaly detector with "<id_fields>" and log predictions in "<output_dir>"
            And I check that the anomaly detector is ready
            Then the anomaly detector has id fields "<id_fields>"
        """
        print(self.test_scenario8.__doc__)
        headers = ["scenario", "kwargs", "id_fields", "output_dir"]
        examples = [
            ['scenario_an_1', '{"data": "data/tiny_kdd.csv",' +
             ' "output": "scenario_an_1/anomaly_scores.csv",' +
             ' "test": "data/test_kdd.csv"}', '["000004", "000005"]',
             'scenario_an_8']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_anomaly.i_create_anomaly_with_id_fields(
                self, id_fields=example["id_fields"],
                output_dir=example["output_dir"])
            test_anomaly.i_check_create_anomaly(self)
            test_anomaly.i_check_anomaly_has_id_fields(
                self, example["id_fields"])



from bigml.api import BigML
api = BigML()
source1_file = "iris_sp_chars.csv"
args = \
{'fields': {'000000': {'name': 'sépal.length', 'optype': 'numeric'},
'000001': {'name': 'sépal&width', 'optype': 'numeric'},
'000002': {'name': 'pétal.length', 'optype': 'numeric'},
'000003': {'name': 'pétal&width\x00', 'optype': 'numeric'},
'000004': {'name': 'spécies',
'optype': 'categorical',
'term_analysis': {'enabled': True}}},
}
source2 = api.create_source(source1_file, args)
api.ok(source2)


# -*- coding: utf-8 -*-
#
# Copyright 2019-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""BigMLer - cluster subcommand processing dispatching

"""


import sys
import os

import bigml.api

import bigmler.utils as u
import bigmler.processing.args as a
import bigmler.processing.sources as ps

from bigmler.defaults import DEFAULTS_FILE
from bigmler.command import get_context
from bigmler.dispatcher import SESSIONS_LOG, clear_log_files

COMMAND_LOG = ".bigmler_source"
DIRS_LOG = ".bigmler_source_dir_stack"
LOG_FILES = [COMMAND_LOG, DIRS_LOG, u.NEW_DIRS_LOG]
MINIMUM_MODEL = "full=false"

SETTINGS = {
    "command_log": COMMAND_LOG,
    "sessions_log": SESSIONS_LOG,
    "dirs_log": DIRS_LOG,
    "default_output": None,
    "defaults_file": DEFAULTS_FILE}


#pylint: disable=locally-disabled,dangerous-default-value
def source_dispatcher(args=sys.argv[1:]):
    """Parses command line and calls the different processing functions

    """

    # If --clear-logs the log files are cleared
    if "--clear-logs" in args:
        clear_log_files(LOG_FILES)

    command_args, _, api, session_file, resume = get_context(args,
                                                             SETTINGS)

    # Selects the action to perform
    if (a.has_train(command_args)
            or (hasattr(command_args, "sources") and
                command_args.sources is not None)
            or command_args.export_fields is not None):
        output_args = a.get_output_args(api, command_args, resume)
        compute_output(**output_args)
    u.log_message("_" * 80 + "\n", log_file=session_file)


def get_metadata(resource, key, default_value):
    """Retrieves from the user_metadata key in the resource the
       given key using default_value as a default

    """
    if ('object' in resource and 'user_metadata' in resource['object'] and
            key in resource['object']['user_metadata']):
        return resource['object']['user_metadata'][key]
    return default_value


def compute_output(api, args):
    """ Creates a dataset using the `training_set`.

    """


    source = None
    fields = None

    # variables from command-line options
    resume = args.resume_
    output = args.output

    path = u.check_dir(output)

    session_file = "%s%s%s" % (path, os.sep, SESSIONS_LOG)
    csv_properties = {}
    # If logging is required set the file for logging
    log = None
    if args.log_file:
        u.check_dir(args.log_file)
        log = args.log_file
        # If --clear_logs the log files are cleared
        clear_log_files([log])

    if args.source_file:
        # source is retrieved from the contents of the given local JSON file
        source, csv_properties, fields = u.read_local_resource(
            args.source_file,
            csv_properties=csv_properties)
    else:
        # source is retrieved from the remote object
        source, resume, csv_properties, fields = ps.source_processing(
            api, args, resume,
            csv_properties=csv_properties,
            session_file=session_file, path=path, log=log)
    if source is not None:
        args.source = bigml.api.get_source_id(source)
    if fields and args.export_fields:
        fields.summary_csv(os.path.join(path, args.export_fields))

    u.print_generated_files(path, log_file=session_file,
                            verbosity=args.verbosity)



# -*- coding: utf-8 -*-
#pylint: disable=locally-disabled,unused-argument,no-member
#
# Copyright 2016-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import os

from bigml.api import check_resource

from bigmler.tests.common_steps import shell_execute, check_rows_equal
from bigmler.tests.world import world, res_filename, ok_, eq_


def i_create_all_td_resources_from_dataset(
    step, test=None, options=None, output=None):
    """Step: I create BigML topic model resources from dataset to test <test>
    with options <options> and log predictions in <output>
    """
    ok_(test is not None and options is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler topic-model " +
               "--minimum-name-terms 0 --dataset " +
               world.dataset['resource'] + " --test " + test +
               " --store --output " + output + " " + options)
    shell_execute(command, output, test=test, options=options)


def i_create_all_td_resources_from_source(
    step, test=None, options=None, output=None):
    """Step: I create BigML topic model resources from source to test <test>
    with options <options> and log predictions in <output>
    """
    ok_(test is not None and options is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler topic-model " +
               "--minimum-name-terms 0 --source " +
               world.source['resource'] + " --test " + test +
               " --store --output " + output + " " + options)
    shell_execute(command, output, test=test, options=options)


def i_create_all_td_resources_from_model(
    step, test=None, options=None, output=None):
    """Step: I create BigML topic model resources from model to test <test>
    with options <options> and log predictions in <output>
    """
    ok_(test is not None and options is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler topic-model " +
               "--minimum-name-terms 0 --topic-model " +
               world.topic_model['resource'] + " --test " + test +
               " --store --output " + output + " " + options)
    shell_execute(command, output, test=test, options=options)


def i_create_topic_distribution_from_model_remote(
    step, test=None, options=None, output=None):
    """Step: I create BigML batch topic distribution from model to test
    <test> with options <options> and log predictions in <output>
    """
    ok_(test is not None and options is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler topic-model --remote" +
               " --minimum-name-terms 0 --topic-model " +
               world.topic_model['resource'] + " --test " + test +
               " --store --output " + output + " " + options)
    shell_execute(command, output, test=test, options=options)


def i_check_create_topic_model(step):
    """Step: I check that the topic model has been created"""
    topic_model_file = os.path.join(world.directory, "topic_models")
    message = None
    try:
        with open(topic_model_file) as handler:
            topic_model = check_resource(handler.readline().strip(),
                                         world.api.get_topic_model)
            world.topic_models.append(topic_model['resource'])
            world.topic_model = topic_model
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def i_check_create_topic_distribution(step):
    """Step: I check that the topic distribution has been created"""
    batch_topic_distribution_file = os.path.join(world.directory,
                                                 "batch_topic_distribution")
    message = None
    try:
        with open(batch_topic_distribution_file) as handler:
            batch_topic_distribution = check_resource(
                handler.readline().strip(),
                world.api.get_batch_topic_distribution)
            world.batch_topic_distribution.append(
                batch_topic_distribution['resource'])
            world.batch_topic_distribution = batch_topic_distribution
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def i_check_create_topic_distributions(step):
    """Step: I check that the remote topic distributions is ready"""
    predictions_lines = 0
    message = None
    try:
        predictions_file = world.output
        with open(predictions_file) as handler:
            predictions_lines = 0
            for _ in handler:
                predictions_lines += 1
            eq_(predictions_lines, world.test_lines,
                msg=f"topic distribution lines: {predictions_lines},"
                    f" test lines: {world.test_lines}")
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def i_check_topic_distributions(step, check_file):
    """Step: the local topic distributions file is like <check_file>"""
    check_file = res_filename(check_file)
    predictions_file = world.output
    message = None
    #pylint: disable=locally-disabled,import-outside-toplevel
    try:
        message = check_rows_equal(predictions_file, check_file)
    except Exception:
        import traceback
        message = traceback.format_exc()
    ok_(message is None, msg=message)


def i_create_topic_model_from_dataset(step, output=None):
    """Step: I create BigML topic model from dataset"""
    ok_(output is not None)
    command = ("bigmler topic-model " +
               "--minimum-name-terms 0 --dataset " +
               world.dataset['resource'] +
               " --store --output " + output)
    shell_execute(command, output)



# -*- coding: utf-8 -*-
#
# Copyright 2020-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Resources management functions

"""


import sys

import bigml.api

from bigmler.utils import (dated, get_url, log_message, check_resource,
                           plural, is_shared,
                           check_resource_error, log_created_resources)
from bigmler.reports import report

from bigmler.resourcesapi.common import set_basic_args, \
    configure_input_fields, update_sample_parameters_args, \
    update_json_args, wait_for_available_tasks

from bigmler.resourcesapi.common import SEED, FIELDS_QS, \
    ALL_FIELDS_QS


def set_topic_model_args(args, name=None, fields=None,
                         topic_model_fields=None):
    """Return topic_model arguments dict

    """
    if name is None:
        name = args.name
    if topic_model_fields is None:
        topic_model_fields = args.topic_fields_

    topic_model_args = set_basic_args(args, name)
    topic_model_args.update({
        "seed": SEED if args.seed is None else args.seed,
        "topicmodel_seed": SEED if args.seed is None else args.seed
    })

    if topic_model_fields and fields is not None:
        input_fields = configure_input_fields(fields, topic_model_fields)
        topic_model_args.update(input_fields=input_fields)
    topic_model_args.update({"sample_rate": args.sample_rate})
    topic_model_args.update({"bigrams": args.bigrams})
    topic_model_args.update({"case_sensitive": args.case_sensitive})
    if args.number_of_topics is not None:
        topic_model_args.update({"number_of_topics": args.number_of_topics})
    if args.term_limit is not None:
        topic_model_args.update({"term_limit": args.term_limit})
    if args.top_n_terms is not None:
        topic_model_args.update({"top_n_terms": args.top_n_terms})
    if args.minimum_name_terms is not None:
        topic_model_args.update({"minimum_name_terms":
                                 args.minimum_name_terms})

    if args.excluded_terms:
        topic_model_args.update({"excluded_terms": args.excluded_terms_})

    topic_model_args = update_sample_parameters_args( \
        topic_model_args, args)

    if 'topic_model' in args.json_args:
        update_json_args(topic_model_args,
                         args.json_args.get('topic_model'),
                         fields)
    return topic_model_args


def create_topic_models(datasets, topic_model_ids, topic_model_args,
                        args, api=None, path=None,
                        session_file=None, log=None):
    """Create remote topic models

    """
    if api is None:
        api = bigml.api.BigML()

    topic_models = topic_model_ids[:]
    existing_topic_models = len(topic_models)
    topic_model_args_list = []
    datasets = datasets[existing_topic_models:]
    # if resuming and all topic models were created, there will
    # be no datasets left
    if datasets:
        if isinstance(topic_model_args, list):
            topic_model_args_list = topic_model_args

        # Only one topic model per command, at present
        number_of_topic_models = 1
        message = dated("Creating %s.\n" %
                        plural("topic model", number_of_topic_models))
        log_message(message, log_file=session_file,
                    console=args.verbosity)

        query_string = FIELDS_QS
        inprogress = []
        for i in range(0, number_of_topic_models):
            wait_for_available_tasks(inprogress,
                                     args.max_parallel_topic_models,
                                     api, "topicmodel")
            if topic_model_args_list:
                topic_model_args = topic_model_args_list[i]

            topic_model = api.create_topic_model(datasets,
                                                 topic_model_args,
                                                 retries=None)
            topic_model_id = check_resource_error( \
                topic_model,
                "Failed to create topic model: ")
            log_message("%s\n" % topic_model_id, log_file=log)
            topic_model_ids.append(topic_model_id)
            inprogress.append(topic_model_id)
            topic_models.append(topic_model)
            log_created_resources("topic_models", path, topic_model_id,
                                  mode='a')

        if args.verbosity:
            if bigml.api.get_status(topic_model)['code'] != bigml.api.FINISHED:
                try:
                    topic_model = check_resource( \
                        topic_model, api.get_topic_model,
                        query_string=query_string,
                        raise_on_error=True)
                except Exception as exception:
                    sys.exit("Failed to get a finished topic model: %s" %
                             str(exception))
                topic_models[0] = topic_model
            message = dated("Topic model created: %s\n" %
                            get_url(topic_model))
            log_message(message, log_file=session_file,
                        console=args.verbosity)
            if args.reports:
                report(args.reports, path, topic_model)

    return topic_models, topic_model_ids


def get_topic_models(topic_model_ids,
                     args, api=None, session_file=None):
    """Retrieves remote topic model in its actual status

    """
    if api is None:
        api = bigml.api.BigML()

    topic_model_id = ""
    topic_models = topic_model_ids
    topic_model_id = topic_model_ids[0]
    message = dated("Retrieving %s. %s\n" %
                    (plural("topic model", len(topic_model_ids)),
                     get_url(topic_model_id)))
    log_message(message, log_file=session_file, console=args.verbosity)
    # only one topic_model at present
    try:
        # we need the whole fields structure when exporting fields
        query_string = FIELDS_QS if not args.export_fields else ALL_FIELDS_QS
        topic_model = check_resource(topic_model_ids[0],
                                     api.get_topic_model,
                                     query_string=query_string,
                                     raise_on_error=True)
    except Exception as exception:
        sys.exit("Failed to get a finished topic model: %s" % \
            str(exception))
    topic_models[0] = topic_model

    return topic_models, topic_model_ids


def set_publish_topic_model_args(args):
    """Set args to publish topic model

    """
    public_topic_model = {}
    if args.public_topic_model:
        public_topic_model = {"private": False}
        if args.model_price:
            public_topic_model.update(price=args.model_price)
        if args.cpp:
            public_topic_model.update(credits_per_prediction=args.cpp)
    return public_topic_model


def update_topic_model(topic_model, topic_model_args,
                       args, api=None, path=None, session_file=None):
    """Updates topic model properties

    """
    if api is None:
        api = bigml.api.BigML()

    message = dated("Updating topic model. %s\n" %
                    get_url(topic_model))
    log_message(message, log_file=session_file,
                console=args.verbosity)
    topic_model = api.update_topic_model(topic_model, \
        topic_model_args)
    check_resource_error(topic_model,
                         "Failed to update topic model: %s"
                         % topic_model['resource'])
    topic_model = check_resource(topic_model,
                                 api.get_topic_model,
                                 query_string=FIELDS_QS,
                                 raise_on_error=True)
    if is_shared(topic_model):
        message = dated("Shared topic model link. %s\n" %
                        get_url(topic_model, shared=True))
        log_message(message, log_file=session_file, console=args.verbosity)
        if args.reports:
            report(args.reports, path, topic_model)

    return topic_model



# -*- coding: utf-8 -*-
#
# Copyright 2015-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""BigMLer - reify processing dispatching

"""


import sys

import bigmler.utils as u

from bigmler.defaults import DEFAULTS_FILE
from bigmler.command import get_context
from bigmler.dispatcher import SESSIONS_LOG, clear_log_files
from bigmler.reify.reify import reify_resources

COMMAND_LOG = ".bigmler_reify"
DIRS_LOG = ".bigmler_reify_dir_stack"
LOG_FILES = [COMMAND_LOG, DIRS_LOG, u.NEW_DIRS_LOG]
DEFAULT_OUTPUT = "reify.py"

SETTINGS = {
    "command_log": COMMAND_LOG,
    "sessions_log": SESSIONS_LOG,
    "dirs_log": DIRS_LOG,
    "default_output": DEFAULT_OUTPUT,
    "defaults_file": DEFAULTS_FILE}


#pylint: disable=locally-disabled,dangerous-default-value
def reify_dispatcher(args=sys.argv[1:]):
    """Parses command line and calls the different processing functions

    """

    # If --clear-logs the log files are cleared
    if "--clear-logs" in args:
        clear_log_files(LOG_FILES)

    command_args, _, api, session_file, _ = get_context(args, SETTINGS)

    message = "Starting reification for %s\n\n" % command_args.resource_id
    u.log_message(message, \
        log_file=session_file, console=command_args.verbosity)
    reify_resources(command_args, api)
    message = "\nReification complete. See the results in %s\n\n" % \
        command_args.output
    u.log_message(message, \
        log_file=session_file, console=command_args.verbosity)
    u.log_message("_" * 80 + "\n", log_file=session_file)

    u.print_generated_files(command_args.output_dir, log_file=session_file,
                            verbosity=command_args.verbosity)
